/*
 * Copyright (C) 1999  Piero Fariselli
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * contacts: Piero Fariselli
 *           e-mail: piero.fariselli@unibo.it
 *           Dept. of Biology
 *           University of Bologna
 *           via Irnerio 42
 *           40126 Bologna
 *           Italy
 */
#include "Alignment.H"
using namespace std;
   
// ************** private methods ***********************

  /** Compute the gapOpen as function of the secondary structure //
    */
  double Alignment::gapOpenf(char c1, char c2) {   
     if((c1 == c2)&&( c1 !=this->baseSymbol)) 
        return this->ssGapOpen*gapOpen;
     else
        return this->gapOpen;
  }

//------------------------------//

   /** scoring function match/mismatch of the two 2 positions and j */
   double Alignment::matchSimMat(int i, int j) {
      return ((double) this->matS[i][j]) ;
   }
//------------------------------//     
//------------------------------//

   /** scoring function match/mismatch of the 2 positins i and j */
   double Alignment::matchSim(int i, int j) {
      double Match;  // sequence scoring matrix  
      double M2;     // structure scoring matrix
      double t=this->weight; // balance weight 0<= t <=1
      
      M2 =  (double) this->matSS[this->SS1[i-1]][this->SS2[j-1]] ;
      Match=  matchSimMat(this->seqNum1[i-1],this->seqNum2[j-1]);
//       System.out.println( i + " "+ j  + " " + M2); 
      return (double) t*Match +(1-t)*M2;
   }
//------------------------------//         


   /** compute the max among 3 values 
       return char COL, DIA, ROW (1,0,-1) depending on the position of the
	   maximum value */
   char Alignment::max(double c, double d, double r) {
      if(d>c)
         if(d>r)
            return DIA;
         else
            return ROW;
      else
         if(c>r)
            return COL;
         else
            return ROW;
   }
//-------------------------------//              



//-------------------------


   /** return an integer matrix which contains the Identity values in the order
       A R N D C Q E G H I  L  K  M  F  P  S  T  W  Y  V B Z X
       0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 2 6 0
     */
   int ** Alignment::setIdentity() {
       int ** mat;
       int i,j;
       int Id[DIMSYMBOL][DIMSYMBOL]={
        {10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,0},
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10},
        };
        mat = new int*[DIMSYMBOL];
        for(i=0; i< DIMSYMBOL; i++) {
           mat[i] = new int [DIMSYMBOL];		
	   for(j=0; j < DIMSYMBOL; j++) 
	      mat[i][j]=Id[i][j];
        }	   
        matSname="Identity";
        return(mat);
   }



//-------------------------

   /** return an integer matrix containing the secondary structure values in the order 
       0 H E C T G S B 3 X D
       0 1 2 3 4 5 6 7 8 9 10 
     */   
   int ** Alignment::setMatSS() {
       // the matrix must be [maxIndexMatSS+1][maxIndexMatSS+1]
       int i,j;
       int ** mat;
       int lm[maxIndexMatSS+1][maxIndexMatSS+1]={
        {10,0,0,0,0,0,0,0,0,0,0},
        {0,10,0,0,0,0,0,0,0,0,0},
        {0,0,10,0,0,0,0,0,0,0,0},
        {0,0,0,10,0,0,0,0,0,0,0},
        {0,0,0,0,10,0,0,0,0,0,0},
        {0,0,0,0,0,10,0,0,0,0,0},
        {0,0,0,0,0,0,10,0,0,0,0},
        {0,0,0,0,0,0,0,10,0,0,0},
        {0,0,0,0,0,0,0,0,10,0,0},
        {0,0,0,0,0,0,0,0,0,10,0},
        {0,0,0,0,0,0,0,0,0,0,10},
        };      
        mat = new int*[maxIndexMatSS+1];
	for(i=0; i<= maxIndexMatSS; i++) {
            mat[i] = new int [maxIndexMatSS+1];
            for(j=0; j <=maxIndexMatSS ; j++)
	              mat[i][j]=lm[i][j];
	}
        return(mat);
   }

   /** return an integer matrix which contains the pam 250 values in the order
       A R N D C Q E G H I  L  K  M  F  P  S  T  W  Y  V B Z X
       0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 2 6 0
     */   
   int** Alignment::setPam250() {
       int ** mat;
       int i,j;
       int pam250[DIMSYMBOL][DIMSYMBOL]={
       {2, -2, 0, 0, -2, 0, 0, 1, -1, -1, -2, -1, -1, -4, 1, 1, 1, -6, -3, 0, 0, 0, 0},
	{-2, 6, 0, -1, -4, 1, -1, -3, 2, -2, -3, 3, 0, -4, 0, 0, -1, 2, -4, -2, -1, 0, 0},
	{0, 0, 2, 2, -4, 1, 1, 0, 2, -2, -3, 1, -2, -4, -1, 1, 0, -4, -2, -2, 2, 1, 0},
	{0, -1, 2, 4, -5, 2, 3, 1, 1, -2, -4, 0, -3, -6, -1, 0, 0, -7, -4, -2, 3, 3, 0},
	{-2, -4, -4, -5, 12, -5, -5, -3, -3, -2, -6, -5, -5, -4, -3, 0, -2, -8, 0, -2, -4, -5, 0},
	{0, 1, 1, 2, -5, 4, 2, -1, 3, -2, -2, 1, -1, -5, 0, -1, -1, -5, -4, -2, 1, 3, 0},
	{0, -1, 1, 3, -5, 2, 4, 0, 1, -2, -3, 0, -2, -5, -1, 0, 0, -7, -4, -2, 2, 3, 0},
	{1, -3, 0, 1, -3, -1, 0, 5, -2, -3, -4, -2, -3, -5, -1, 1, 0, -7, -5, -1, 0, -1, 0},
	{-1, 2, 2, 1, -3, 3, 1, -2, 6, -2, -2, 0, -2, -2, 0, -1, -1, -3, 0, -2, 1, 2, 0},
	{-1, -2, -2, -2, -2, -2, -2, -3, -2, 5, 2, -2, 2, 1, -2, -1, 0, -5, -1, 4, -2, -2, 0},
	{-2, -3, -3, -4, -6, -2, -3, -4, -2, 2, 6, -3, 4, 2, -3, -3, -2, -2, -1, 2, -3, -3, 0},
	{-1, 3, 1, 0, -5, 1, 0, -2, 0, -2, -3, 5, 0, -5, -1, 0, 0, -3, -4, -2, 1, 0, 0},
	{-1, 0, -2, -3, -5, -1, -2, -3, -2, 2, 4, 0, 6, 0, -2, -2, -1, -4, -2, 2, -2, -2, 0},
	{-4, -4, -4, -6, -4, -5, -5, -5, -2, 1, 2, -5, 0, 9, -5, -3, -3, 0, 7, -1, -5, -5, 0},
	{1, 0, -1, -1, -3, 0, -1, -1, 0, -2, -3, -1, -2, -5, 6, 1, 0, -6, -5, -1, -1, 0, 0},
	{1, 0, 1, 0, 0, -1, 0, 1, -1, -1, -3, 0, -2, -3, 1, 2, 1, -2, -3, -1, 0, 0, 0},
	{1, -1, 0, 0, -2, -1, 0, 0, -1, 0, -2, 0, -1, -3, 0, 1, 3, -5, -3, 0, 0, -1, 0},
	{-6, 2, -4, -7, -8, -5, -7, -7, -3, -5, -2, -3, -4, 0, -6, -2, -5, 17, 0, -6, -5, -6, 0},
	{-3, -4, -2, -4, 0, -4, -4, -5, 0, -1, -1, -4, -2, 7, -5, -3, -3, 0, 10, -2, -3, -4, 0},
	{0, -2, -2, -2, -2, -2, -2, -1, -2, 4, 2, -2, 2, -1, -1, -1, 0, -6, -2, 4, -2, -2, 0},
	{0, -1, 2, 3, -4, 1, 2, 0, 1, -2, -3, 1, -2, -5, -1, 0, 0, -5, -3, -2, 2, 2, 0},
	{0, 0, 1, 3, -5, 3, 3, -1, 2, -2, -3, 0, -2, -5, 0, 0, -1, -6, -4, -2, 2, 3, 0},
	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
        };      
        mat = new int*[DIMSYMBOL];
        for(i=0; i< DIMSYMBOL; i++) {
               mat[i] = new int [DIMSYMBOL];
               for(j=0; j < DIMSYMBOL; j++)
                    mat[i][j]=pam250[i][j];
        }
        matSname="PAM250";
        return(mat);
   }



  /** set blosum62
       A R N D C Q E G H I  L  K  M  F  P  S  T  W  Y  V B Z X
       0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 2 6 0
     */   
   int** Alignment::setBlosum62() {
       int ** mat;
       int i,j;
       int blosum62[DIMSYMBOL][DIMSYMBOL]={
	       {4,-1,-2,-2,0,-1,-1,0,-2,-1,-1,-1,-1,-2,-1,1,0,-3,-2,0,-2,-1,0},
	       {-1,5,0,-2,-3,1,0,-2,0,-3,-2,2,-1,-3,-2,-1,-1,-3,-2,-3,-1,0,-1},
	       {-2,0,6,1,-3,0,0,0,1,-3,-3,0,-2,-3,-2,1,0,-4,-2,-3,3,0,-1},
	       {-2,-2,1,6,-3,0,2,-1,-1,-3,-4,-1,-3,-3,-1,0,-1,-4,-3,-3,4,1,-1},
	       {0,-3,-3,-3,9,-3,-4,-3,-3,-1,-1,-3,-1,-2,-3,-1,-1,-2,-2,-1,-3,-3,-2},
	       {-1,1,0,0,-3,5,2,-2,0,-3,-2,1,0,-3,-1,0,-1,-2,-1,-2,0,3,-1},
	       {-1,0,0,2,-4,2,5,-2,0,-3,-3,1,-2,-3,-1,0,-1,-3,-2,-2,1,4,-1},
	       {0,-2,0,-1,-3,-2,-2,6,-2,-4,-4,-2,-3,-3,-2,0,-2,-2,-3,-3,-1,-2,-1},
	       {-2,0,1,-1,-3,0,0,-2,8,-3,-3,-1,-2,-1,-2,-1,-2,-2,2,-3,0,0,-1},
	       {-1,-3,-3,-3,-1,-3,-3,-4,-3,4,2,-3,1,0,-3,-2,-1,-3,-1,3,-3,-3,-1},
	       {-1,-2,-3,-4,-1,-2,-3,-4,-3,2,4,-2,2,0,-3,-2,-1,-2,-1,1,-4,-3,-1},
	       {-1,2,0,-1,-3,1,1,-2,-1,-3,-2,5,-1,-3,-1,0,-1,-3,-2,-2,0,1,-1},
	       {-1,-1,-2,-3,-1,0,-2,-3,-2,1,2,-1,5,0,-2,-1,-1,-1,-1,1,-3,-1,-1},
	       {-2,-3,-3,-3,-2,-3,-3,-3,-1,0,0,-3,0,6,-4,-2,-2,1,3,-1,-3,-3,-1},
	       {-1,-2,-2,-1,-3,-1,-1,-2,-2,-3,-3,-1,-2,-4,7,-1,-1,-4,-3,-2,-2,-1,-2},
	       {1,-1,1,0,-1,0,0,0,-1,-2,-2,0,-1,-2,-1,4,1,-3,-2,-2,0,0,0},
	       {0,-1,0,-1,-1,-1,-1,-2,-2,-1,-1,-1,-1,-2,-1,1,5,-2,-2,0,-1,-1,0},
	       {-3,-3,-4,-4,-2,-2,-3,-2,-2,-3,-2,-3,-1,1,-4,-3,-2,11,2,-3,-4,-3,-2},
	       {-2,-2,-2,-3,-2,-1,-2,-3,2,-1,-1,-2,-1,3,-3,-2,-2,2,7,-1,-3,-2,-1},
	       {0,-3,-3,-3,-1,-2,-2,-3,-3,3,1,-2,1,-1,-2,-2,0,-3,-1,4,-3,-2,-1},
	       {-2,-1,3,4,-3,0,1,-1,0,-3,-4,0,-3,-3,-2,0,-1,-4,-3,-3,4,1,-1},
	       {-1,0,0,1,-3,3,4,-2,0,-3,-3,1,-1,-3,-1,0,-1,-3,-2,-2,1,4,-1},
	       {0,-1,-1,-1,-2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-2,0,0,-2,-1,-1,-1,-1,-1}};      
        mat = new int*[DIMSYMBOL];
        for(i=0; i< DIMSYMBOL; i++) {
               mat[i] = new int [DIMSYMBOL];
               for(j=0; j < DIMSYMBOL; j++)
                    mat[i][j]=blosum62[i][j];
        }
        matSname="BLOSUM62";
        return(mat);
   }

//----------------

//-------------------------
   /** return an int vector coding for the sequence in the order
       A R N D C Q E G H I  L  K  M  F  P  S  T  W  Y  V B Z X
       0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 2 6 0
     */   
   int* Alignment::SeqByte(Sequence s, int len) {
      int i; 
      int* b;
      b= new int[len];
      for(i=0; i<len; i++) { 
         switch (s.charAt(i)) {
            case 'A' : b[i]=0 ; break;
            case 'R' : b[i]=1 ; break;
            case 'N' : b[i]=2 ; break;
            case 'D' : b[i]=3 ; break;
            case 'C' : b[i]=4 ; break;
            case 'Q' : b[i]=5 ; break;
            case 'E' : b[i]=6 ; break;
            case 'G' : b[i]=7 ; break;
            case 'H' : b[i]=8 ; break;
            case 'I' : b[i]=9 ; break;
            case 'L' : b[i]=10; break;
            case 'K' : b[i]=11; break;
            case 'M' : b[i]=12; break;
            case 'F' : b[i]=13; break;
            case 'P' : b[i]=14; break;
            case 'S' : b[i]=15; break;
            case 'T' : b[i]=16; break;
            case 'W' : b[i]=17; break;
            case 'Y' : b[i]=18; break;
            case 'V' : b[i]=19; break;
            case 'B' : b[i]=2 ; break; // residui anomali
            case 'Z' : b[i]=6 ; break; //  residui anomali
            case 'X' : b[i]=0 ; break; //  residui anomali
          default :  break;
         }
//      System.out.print(b[i]+"\n");
      }  
      return(b); 
   }      

//-------------------------


//-------------------------
   /** return an int vector in the order 
       0 H E C T G S B 3 X D
       0 1 2 3 4 5 6 7 8 9 10 
     */
   int * Alignment::SeqByteSS(char * s, int lenS, int len) {
      int i;
      int * b;
      b= new int[len];
      if(lenS != len) { 
         cout << "########Different length! " << lenS <<" "<<len << endl;
         for(i=0; i<len; i++)
            b[i]=maxIndexMatSS;
      }
      else {
         for(i=0; i<lenS; i++) {
            switch (s[i]) {
               case  '0' : b[i]=0 ; break;
               case  'H' : b[i]=1 ; break;
               case  'E' : b[i]=2 ; break;
               case  'C' : b[i]=3 ; break;
               case  'T' : b[i]=4 ; break;
               case  'G' : b[i]=5 ; break;
               case  'S' : b[i]=6 ; break;
               case  'B' : b[i]=7 ; break;
               case  '3' : b[i]=8 ; break;
               case  'X' : b[i]=9 ; break;
               case  'D' : b[i]=maxIndexMatSS; break;
               default   : break;
            }
//      System.out.print(b[i]+"\n");
         }
      }   
//      System.out.println(s);
      return(b);
   }


//-------------------------------// 
  /**  compute the aligned sequences */
   void Alignment::mkAlignedSequences() {
      int mat,i, j,k;
      int maxlen=this->maxR+this->maxC+1;
      char * s1,*s1ss,*s2ss, *s2;
 
      i=this->maxR-1; j=this->maxC-1;
 
      s2ss= new char[maxlen+1]; 
      s1ss= new char[maxlen+1];
      s1=  new char[maxlen+1];
      s2=  new char[maxlen+1];
      mat=0;
      s2ss[maxlen]=s1ss[maxlen]=s1[maxlen]=s2[maxlen]='\0';
      k=maxlen-1;
      while((i>0)&&(j>0)) {
         switch (mat) {
            case 0:  switch (matBack[mat][i][j]) {   // diagonal
                       case DIA:  s1[k]=S1.charAt(i-1); s1ss[k]=S1.ssAt(i-1);
				  s2[k]=S2.charAt(j-1); s2ss[k]=S2.ssAt(j-1);
                                  i--;j--; k--;mat =0; break;
                       case ROW:  mat =1; break;
                                  
                       case COL:  mat =2; break;
                                  
                     }break;
            case 1:  switch (matBack[mat][i][j]) {   // row
                       case DIA:  mat =0; break;
                       case ROW: mat =1; break;
                     }
		     s1[k]=this->gapSymbol; s2[k]=S2.charAt(j-1);
		     s1ss[k]=this->gapSymbol;s2ss[k]=S2.ssAt(j-1);
                     j--; k--;break;
            case 2:  switch (matBack[mat][i][j]) {   // column
                       case DIA:  mat =0; break;
                       case COL: mat =2; break;
                     }
		     s1[k]=S1.charAt(i-1); s2[k]=this->gapSymbol;
		     s1ss[k]=S1.ssAt(i-1); s2ss[k]=this->gapSymbol;
                     i--; k--; break;
         } 
      }
      while(i>0) { s1[k]=S1.charAt(i-1); s2[k]=this->gapSymbol;
	           s1ss[k]=S1.ssAt(i-1); s2ss[k]=this->gapSymbol;
                   i--;k--;}
      while(j>0) { s1[k]=this->gapSymbol; s2[k]=S2.charAt(j-1);
                   s1ss[k]=this->gapSymbol;s2ss[k]=S2.ssAt(j-1);
                   j--;k--;}
      k++;
      
//      cout << &s2[k]<<endl;
//      cout << &s2ss[k]<<endl;
      this->S1aligned=Sequence(S1.getName(),&s1[k],&s1ss[k]);
      this->S2aligned=Sequence(S2.getName(),&s2[k],&s2ss[k]);           

      return;
   }


//-------------------------------//

//-------------------------------//
  /** compute the best alignment */
   void Alignment::mkAlignedSequences(int br, int bc) {
      int mat,i, j,k;
      int maxlen=this->maxR+this->maxC+1;
      char * s1,*s1ss,*s2ss, *s2;

      i=this->maxR-1; j=this->maxC-1;

      s2ss= new char[maxlen+1];
      s1ss= new char[maxlen+1];
      s1=  new char[maxlen+1];
      s2=  new char[maxlen+1];
      mat=0;
      s2ss[maxlen]=s1ss[maxlen]=s1[maxlen]=s2[maxlen]='\0';
      k=maxlen-1;

      while(i>br) { s1[k]=S1.charAt(i-1); s2[k]=this->gapSymbol;
                   s1ss[k]=S1.ssAt(i-1); s2ss[k]=this->gapSymbol;
                   i--;k--;}
      while(j>bc) { s1[k]=this->gapSymbol; s2[k]=S2.charAt(j-1);
                   s1ss[k]=this->gapSymbol;s2ss[k]=S2.ssAt(j-1);
                   j--;k--;}

      while((i>0)&&(j>0)) {
         switch (mat) {
            case 0:  switch (matBack[mat][i][j]) {   // diagonal
                       case DIA:  s1[k]=S1.charAt(i-1); s1ss[k]=S1.ssAt(i-1);
                                  s2[k]=S2.charAt(j-1); s2ss[k]=S2.ssAt(j-1);
                                  i--;j--; k--;mat =0; break;
                       case ROW:  mat =1; break;

                       case COL:  mat =2; break;

                     }break;
            case 1:  switch (matBack[mat][i][j]) {   // row
                       case DIA:  mat =0; break;
                       case ROW: mat =1; break;
                     }
                     s1[k]=this->gapSymbol; s2[k]=S2.charAt(j-1);
                     s1ss[k]=this->gapSymbol;s2ss[k]=S2.ssAt(j-1);
                     j--; k--;break;
            case 2:  switch (matBack[mat][i][j]) {   // column
                       case DIA:  mat =0; break;
                       case COL: mat =2; break;
                     }
                     s1[k]=S1.charAt(i-1); s2[k]=this->gapSymbol;
                     s1ss[k]=S1.ssAt(i-1); s2ss[k]=this->gapSymbol;
                     i--; k--; break;
         }
      }
      while(i>0) { s1[k]=S1.charAt(i-1); s2[k]=this->gapSymbol;
                   s1ss[k]=S1.ssAt(i-1); s2ss[k]=this->gapSymbol;
                   i--;k--;}
      while(j>0) { s1[k]=this->gapSymbol; s2[k]=S2.charAt(j-1);
                   s1ss[k]=this->gapSymbol;s2ss[k]=S2.ssAt(j-1);
                   j--;k--;}
      k++;

//      cout << &s2[k]<<endl;
//      cout << &s2ss[k]<<endl;
      this->S1aligned=Sequence(S1.getName(),&s1[k],&s1ss[k]);
      this->S2aligned=Sequence(S2.getName(),&s2[k],&s2ss[k]);

      return;
   }


//-------------------------------//



//##################################//
//  Public methods //
   
// Builders
//   /** empty */
//   Alignment() { }
   /** two sequence builder */
   Alignment::Alignment(Sequence s1, Sequence s2) {
      int k,i;
      this->S1=Sequence(s1);
      this->S2=Sequence(s2);
      this->maxR=s1.getLen()+1;
      this->maxC=s2.getLen()+1;
      this->matMR = new double * [maxR]; for(i=0;i<maxR;i++) {this->matMR[i]= new double[maxC];};
      this->matI  = new double * [maxR]; for(i=0;i<maxR;i++) {this->matI[i]= new double[maxC];};
      this->matD  = new double * [maxR]; for(i=0;i<maxR;i++) {this->matD[i]= new double[maxC];};
      this->matBack = new char** [3]; 
                      for(k=0;k<3;k++) { 
	                  this->matBack[k]= new char * [maxR];
                          for(i=0;i<maxR;i++)
			        this->matBack[k][i] = new char [maxC];
		       }
      this->matSS = setMatSS();
      this->matS = setPam250();
//      this->matS = setIdentity();
      this->seqNum1=SeqByte(this->S1,this->maxR-1);
      this->SS1=SeqByteSS(this->S1.getStringSS(),this->S1.getLengthSS(),this->maxR-1);
      this->seqNum2=SeqByte(this->S2,this->maxC-1);
      this->SS2=SeqByteSS(this->S2.getStringSS(),this->S2.getLengthSS(),this->maxC-1);
      this->weight=1.0;
      this->gapOpen=-12.0;
      this->gapExt=-2.0;
      this->ssGapOpen=2.5;

//      System.out.println("ROW "+maxR+"COL "+ maxC);
   }

//---------------------------//



   Alignment::~Alignment() {
   }

// *********** methods  ************
/////   alignment methods ////////////////

//------------------------------------------//                            

   /** Gotoh's alignment with max similarity */
   double Alignment::gotohAlSim() {
      int i, j;
      char decision; // where to go
      double dtmp, ctmp, rtmp; // temporary values
      double t;                // temp varible
      double Infinito=-10000.0;
 
    // border conditions
      // first cells      
      matMR[0][0]=0.0; matBack[0][0][0]=matBack[1][0][0]=matBack[2][0][0]=ZERO; t=gapOpen;
      // 
      /** first row */
      for(j=1; j< this->maxC; j++ ){   
         t+=gapExt; 
         matMR[0][j]=t;
         matI[0][j]= 0.0; 
         matD[0][j]= Infinito;
//         matD[0][j]= t+ gapOpen;
         matBack[0][0][j]=matBack[1][0][j]=matBack[2][0][j]=ROW;
      }
      // recurrence 
      t=gapOpen;
      for(i=1; i< this->maxR; i++ ){
         t+=gapExt; 
         matMR[i][0]=t;  // first column
         matD[i][0]=0.0;
         matI[i][0]=Infinito;
//         matI[i][0]=t+gapOpen;
         matBack[0][i][0]=matBack[1][i][0]=matBack[2][i][0]=COL;
         for(j=1; j< this->maxC; j++ ){
            if(matI[i][j-1]>matMR[i][j-1]+gapOpen) {
                matI[i][j]=matI[i][j-1]+gapExt;
                matBack[1][i][j]=ROW;
            }
            else {
                matI[i][j]=matMR[i][j-1]+gapOpen+gapExt;
                matBack[1][i][j]=DIA;
            }
            if(matD[i-1][j]>matMR[i-1][j]+gapOpen) {
                matD[i][j]=matD[i-1][j]+gapExt;
                matBack[2][i][j]=COL;
            }
            else {
                matD[i][j]=matMR[i-1][j]+gapOpen+gapExt; 
                matBack[2][i][j]=DIA;
            } 
            dtmp=matMR[i-1][j-1]+matchSim(i,j);
//            dtmp=matMR[i-1][j-1]+matchSimMat(this->seqNum1[i-1],this->seqNum2[j-1]);
            rtmp=matI[i][j];
            ctmp=matD[i][j];
            decision=max(ctmp,dtmp,rtmp);
//            System.out.println(" C " + ctmp + " D "+dtmp+ " R "+rtmp + " "+gapOpen);
            switch (decision) {
               case DIA:  matMR[i][j] = dtmp; matBack[0][i][j]=DIA; break;
               case ROW:  matMR[i][j] = rtmp; matBack[0][i][j]=ROW; break;
               case COL:  matMR[i][j] = ctmp; matBack[0][i][j]=COL;
            }

         }
      }
      this->mkAlignedSequences();
      return(matMR[maxR-1][maxC-1]);
   }
//------------------------------------------//            



   /** Gotoh's algorithm using weighted score and different gapOpen values */
   double Alignment::gotohAlSimGapWeight() {
      int i, j;
      char decision; // where to go
      double dtmp, ctmp, rtmp; //  temp values
      double t;                // temp value
      double Infinito=-10000.0;
      double locGap;

    // border conditions
      // first cells
      matMR[0][0]=0.0; matBack[0][0][0]=matBack[1][0][0]=matBack[2][0][0]=ZERO; t=gapOpen;
      //
      /** first row */
      for(j=1; j< this->maxC; j++ ){   
         t+=gapExt;
         matMR[0][j]=t;
         matI[0][j]= 0.0;
         matD[0][j]= Infinito;
//         matD[0][j]= t+ gapOpen;
         matBack[0][0][j]=matBack[1][0][j]=matBack[2][0][j]=ROW;
      }
      // start recurrence
      t=gapOpen;
      for(i=1; i< this->maxR; i++ ){
         t+=gapExt;
         matMR[i][0]=t;  // first column
         matD[i][0]=0.0;
         matI[i][0]=Infinito;
//         matI[i][0]=t+gapOpen;
         matBack[0][i][0]=matBack[1][i][0]=matBack[2][i][0]=COL;
         for(j=1; j< this->maxC; j++ ){
            if(i < this->maxR -1) { locGap=gapOpenf(S1.ssAt(i-1),S1.ssAt(i)); }
            else locGap=gapOpen;
            if(matI[i][j-1]>matMR[i][j-1]+locGap) {
                matI[i][j]=matI[i][j-1]+gapExt;
                matBack[1][i][j]=ROW;
            }
            else {
                matI[i][j]=matMR[i][j-1]+locGap+gapExt;
                matBack[1][i][j]=DIA;
            }
            if(j < this->maxC -1) { locGap=gapOpenf(S2.ssAt(j-1),S2.ssAt(j)); }
            else locGap=gapOpen;
            if(matD[i-1][j]>matMR[i-1][j]+locGap) {
                matD[i][j]=matD[i-1][j]+gapExt;
                matBack[2][i][j]=COL;
            }
            else {
                matD[i][j]=matMR[i-1][j]+locGap+gapExt;
                matBack[2][i][j]=DIA;
            }
            dtmp=matMR[i-1][j-1]+matchSim(i,j);
//            dtmp=matMR[i-1][j-1]+matchSimMat(this->seqNum1[i-1],this->seqNum2[j-1]);
            rtmp=matI[i][j];
            ctmp=matD[i][j];
            decision=max(ctmp,dtmp,rtmp);
//            System.out.println(" C " + ctmp + " D "+dtmp+ " R "+rtmp + " "+gapOpen);
            switch (decision) {
               case DIA:  matMR[i][j] = dtmp; matBack[0][i][j]=DIA; break;
               case ROW:  matMR[i][j] = rtmp; matBack[0][i][j]=ROW; break;
               case COL:  matMR[i][j] = ctmp; matBack[0][i][j]=COL;
            }

         }
      }
      this->bestR=maxR-1;
      this->bestC=maxC-1;
      this->mkAlignedSequences();
      return(matMR[maxR-1][maxC-1]);
   }
//------------------------------------------//


   /** semiglobal alignment: does not weight the end gaps*/
   double Alignment::gotohAlSimGapWeight0() {
      int i, j;
      char decision; //  where to go
      double dtmp, ctmp, rtmp; // temp values
      double t;                // temp value      
      double Infinito=-10000.0;
      double locGap;
      int bestR,bestC;
      double bestVal;
 
    // border conditions
      // first cells      
      matMR[0][0]=0.0; matBack[0][0][0]=matBack[1][0][0]=matBack[2][0][0]=ZERO; t=gapOpen;
      // 
      /** first row */
      for(j=1; j< this->maxC; j++ ){ 
         t+=gapExt; 
         matMR[0][j]=0;
         matI[0][j]= 0.0; 
         matD[0][j]= Infinito;
//         matD[0][j]= t+ gapOpen;
         matBack[0][0][j]=matBack[1][0][j]=matBack[2][0][j]=ROW;
      }
      // start recurrence
      bestVal=0.0;
      bestR = this->maxR -1;
      bestC = this->maxC -1;
      t=gapOpen;
      for(i=1; i< this->maxR; i++ ){
         t+=gapExt; 
//         matMR[i][0]=0.0;  // first column
         matMR[i][0]=0;  // first column
         matD[i][0]=0.0;
         matI[i][0]=Infinito;
//         matI[i][0]=t+gapOpen;
         matBack[0][i][0]=matBack[1][i][0]=matBack[2][i][0]=COL;
         for(j=1; j< this->maxC; j++ ){
            if(i < this->maxR -1) { locGap=gapOpenf(S1.ssAt(i-1),S1.ssAt(i)); }
            else locGap=gapOpen;
            if(matI[i][j-1]>matMR[i][j-1]+locGap) {
                matI[i][j]=matI[i][j-1]+gapExt;
                matBack[1][i][j]=ROW;
            }
            else {
                matI[i][j]=matMR[i][j-1]+locGap+gapExt;
                matBack[1][i][j]=DIA;
            }
            if(j < this->maxC -1) { locGap=gapOpenf(S2.ssAt(j-1),S2.ssAt(j)); }
            else locGap=gapOpen;
            if(matD[i-1][j]>matMR[i-1][j]+locGap) {
                matD[i][j]=matD[i-1][j]+gapExt;
                matBack[2][i][j]=COL;
            }
            else {
                matD[i][j]=matMR[i-1][j]+locGap+gapExt; 
                matBack[2][i][j]=DIA;
            } 
            dtmp=matMR[i-1][j-1]+matchSim(i,j);
//            dtmp=matMR[i-1][j-1]+matchSimMat(this->seqNum1[i-1],this->seqNum2[j-1]);
            rtmp=matI[i][j];
            ctmp=matD[i][j];
            decision=max(ctmp,dtmp,rtmp);
//            System.out.println(" C " + ctmp + " D "+dtmp+ " R "+rtmp + " "+gapOpen);
            switch (decision) {
               case DIA:  matMR[i][j] = dtmp; matBack[0][i][j]=DIA; break;
               case ROW:  matMR[i][j] = rtmp; matBack[0][i][j]=ROW; break;
               case COL:  matMR[i][j] = ctmp; matBack[0][i][j]=COL;
            }
            if((i == (this->maxR -1))||(j == (this->maxC -1))) 
                if(matMR[i][j]>bestVal) {
                   bestVal=matMR[i][j];
                   bestR = i;
                   bestC = j;
                }

         }
      }
      this->bestR=bestR;
      this->bestC=bestC;
      this->mkAlignedSequences(bestR,bestC);
      return(matMR[bestR][bestC]);
   }
//------------------------------------------//       



// parameter settings //
//
   /**  read an integer matrix in the order 
       A R N D C Q E G H I  L  K  M  F  P  S  T  W  Y  V B Z X
       0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 2 6 0
     */

   int ** Alignment::readResidueMatrix(char * filename) {
      int i, j;
      int **mat;
      ifstream f(filename);
      
      if(!f) {
        cout << "### WARNING!!!: Can't open file "<< filename <<endl;;
        return (NULL);
      }

      mat = new int*[DIMSYMBOL];
      for(i=0; i< DIMSYMBOL; i++) {
         mat[i] = new int [DIMSYMBOL];		
      }	   
      i=j=0;
      for(i=0; i <DIMSYMBOL;i++)
        for(j=0; j<=i;j++)   {     		 
	   f>>mat[j][i];
           mat[i][j]=mat[j][i];
	}
           	   
      f.close();
      matSname = new char [strlen(filename)+11];
      sprintf(matSname,"InputFile: %s",filename);
      this->matS=mat;
      return(mat);

   }
//-------------------------------------//


   /** read an integer matrix in the order
       0 H E C T G S B 3 X D
       0 1 2 3 4 5 6 7 8 9 10 
     */

   int ** Alignment::readSSMatrix(char * filename) {
      int i, j;
      int **mat;
      ifstream f(filename);
      
      if(!f) {
        cout << "### WARNING!!!: Can't open file "<< filename <<endl;;
        return (NULL);
      }

      mat = new int*[maxIndexMatSS+1];
      for(i=0; i<= maxIndexMatSS; i++) {
          mat[i] = new int [maxIndexMatSS+1];
      }
      i=j=0;
      for(i=0; i <=maxIndexMatSS;i++)
        for(j=0; j<=i;j++)   {     		 
	   f>>mat[j][i];
           mat[i][j]=mat[j][i];
	}
           	   
      f.close();
      this->matSS=mat;
      return(mat);

   }
//-------------------------------------//


   /** set the weight value  */
   void Alignment::setWeight(double w) {
      if((w>=0)&&(w<=1)) 
          this->weight = w;
   }
//----------------------------//

   /** set the score matrix */
   void Alignment::setScoreMatrix(char * s) {
      char c = toupper(s[0]);	   
      if(strcasecmp(s,"BLOSUM62")==0){
         this->matS = setBlosum62();
         return;
      }
      if(strcasecmp(s,"PAM250")==0){
         this->matS = setPam250();
         return;
      }
      if(strcasecmp(s,"Identity")==0){
        this->matS = setIdentity();
         return;
      }
      switch (c) {
         case 'B' :this->matS = setBlosum62(); break; 	      
         case 'P' :this->matS = setPam250(); break; 	      
         case 'I' :this->matS = setIdentity(); break; 	      
      }
      return;
   }

//----------------------------//


   /** set gap open and extension */
   void Alignment::setGap(double gapOpen, double gapExt) {
      this->gapOpen = gapOpen;
      this->gapExt = gapExt;
   }
//----------------------------//


   /** set gap extension */
   void Alignment::setGapExt(double gap) {
      this->gapExt = gap;
   }
//----------------------------//

   /** set the multiplicative open value for the secondary structures */
   void Alignment::setSSGapOpen(double gap) {
      if(gap >=1.0)
         this->ssGapOpen = gap;
   }
//----------------------------//

   /** set the gap open */
   void Alignment::setGapOpen(double gap) {
      this->gapOpen = gap;
   }
//----------------------------//

   /** set the gap value  */
   void Alignment::setGap(double gap) {
      this->gap = gap;
   }
//----------------------------//

// *************** prints **************************

   /** print score value */
   double Alignment::printScore() {
      bestR=this->bestR;
      bestC=this->bestC;      
      cout <<"ALIGNMENT SCORE = "<<this->matMR[bestR][bestC]<<" with MATRIX "<<this->matSname<<endl;;
      return(this->matMR[bestR][bestC]);
   }

   /** print Isert matrix  */
   void Alignment::printMatIns() {
      int i, j;

      for(i=0; i< this->maxR; i++ ) {
         for(j=0; j< this->maxC; j++ )
            cout << " " << matI[i][j];
         cout <<"\n";
      }
      return;
   }
//---------------------------------//

   /** print deletion matrix */
   void Alignment::printMatDel() {
      int i, j;

      for(i=0; i< this->maxR; i++ ) {
         for(j=0; j< this->maxC; j++ )
           cout << " " << matD[i][j];
        cout <<"\n";
      }
      return;
   }
//---------------------------------//

   /** print score matrix */
   void Alignment::printMatVal() {
      int i, j;

      for(i=0; i< this->maxR; i++ ) {
         for(j=0; j< this->maxC; j++ )
            cout << " " << matMR[i][j];
         cout <<"\n";
      }
      return;
   }
//---------------------------------//

   /** print backtrace matrix */
   void Alignment::printMatBack() {
      int k, i, j;

      for(k=0; k<3 ; k++) {
         cout << " MATRIX BACK  "<< k << "  \n";
         for(i=0; i< this->maxR; i++ ) {
            for(j=0; j< this->maxC; j++ )
	       cout << matBack[k][i][j];
            cout <<"\n";
         }
      }
      return;
   }
//--------------------------//


//-------------------------------//
   /** print the alignment */

   void Alignment::printMapSS() {
      int pos,i, len;
	   

      len=S1aligned.getLen()-1; 
      
      i=0;
      pos=1;
      while(len>=i) {
         if(S1aligned.charAt(i) != this->gapSymbol){
//	    cout << S1aligned.charAt(i);
            if(S2aligned.charAt(i) == this->gapSymbol ) 
                cout <<pos<<"\t-\n";
            else 
                cout <<pos<<"\t"<<S2aligned.ssAt(i)<<"\n";
	    pos++;
         }
         i++;
      }
      return;
   }

//-------------------------------//





//-------------------------------//
   /** print the alignment in columns of max lenght */
   void Alignment::printAllAlignment(int max) {
      int i;
      int shift;
      int len;
      int printlen;
      float identiySeq=0.0f;
 
 
      len=this->S1aligned.getLen(); 
      for(i=0;i<len;i++) 
         if((S1aligned.charAt(i)==S2aligned.charAt(i))&&(S2aligned.charAt(i)!=gapSymbol))
              identiySeq++;
        
//      stxt=s1+" \n"+s2+"\n";  
      
      cout<<" Parameter: gapOpen "<<this->gapOpen <<"   gapExtention " << this->gapExt<<"\n"; 
      cout<<" Parameter: gapOpen for Secondary Structure "<<this->gapOpen*this->ssGapOpen <<"\n"; 
      cout<<" Secondary structure weight = "<<this->weight<< " |  [1 =Only RESIDUE / 0 =Only SS]\n";
      identiySeq/=len;
      cout<<"Sequece Identity = "<<identiySeq<< "\n";
      cout <<"S1 = "<<S1.getName()<<"\n";
      cout <<"S2 = "<<S2.getName()<<"\n";
//
      shift=0;
      while(len>0) {
         printlen=(len>max)?max-1:len-1;
         cout<<"\nS1   ";
         for(i=0; i< printlen; i++)
	    cout<<S1aligned.charAt(i+shift);
	 cout<<S1aligned.charAt(i+shift)<<"\n";
         cout<<"SS1  ";
         for(i=0; i< printlen; i++)
            cout<<S1aligned.ssAt(i+shift);
         cout<<S1aligned.ssAt(i+shift)<<"\n";
         cout<<"S2   ";
         for(i=0; i< printlen; i++)
            cout<<S2aligned.charAt(i+shift);
         cout<<S2aligned.charAt(i+shift)<<"\n";
         cout<<"SS2  ";
         for(i=0; i< printlen; i++)
            cout<<S2aligned.ssAt(i+shift);
         cout<<S2aligned.ssAt(i+shift)<<"\n";
         shift+=max;
         len-=max;
      }
      return;
   }

//-------------------------------//


//-------------------------------//
   /** print only the aligned sequences only  */
   void Alignment::printAlignment(int max) {
      int i;
      int shift;
      int len;
      int printlen;
      float identiySeq=0.0f;
 
 
      len=this->S1aligned.getLen(); 
      for(i=0;i<len;i++) 
         if((S1aligned.charAt(i)==S2aligned.charAt(i))&&(S2aligned.charAt(i)!=gapSymbol))
              identiySeq++;
        
//      stxt=s1+" \n"+s2+"\n";  
      
      cout<<" Parameter: gapOpen "<<this->gapOpen <<"   gapExtention " << this->gapExt<<"\n"; 
      cout<<" Parameter: gapOpen for Secondary Structure "<<this->gapOpen*this->ssGapOpen <<"\n"; 
      cout<<" Secondary structure weight = "<<this->weight<< " |  [1 =Only RESIDUE / 0 =Only SS]\n";
      identiySeq/=len;
      cout<<"Sequece Identity = "<<identiySeq<< "\n";
      cout <<"S1 = "<<S1.getName()<<"\n";
      cout <<"S2 = "<<S2.getName()<<"\n";

// 
      shift=0;
      while(len>0) {
         printlen=(len>max)?max-1:len-1;
         cout<<"\nS1   ";
         for(i=0; i< printlen; i++)
	    cout<<S1aligned.charAt(i+shift);
	 cout<<S1aligned.charAt(i+shift)<<"\n";
         cout<<"S2   ";
         for(i=0; i< printlen; i++)
            cout<<S2aligned.charAt(i+shift);
         cout<<S2aligned.charAt(i+shift)<<"\n";
         shift+=max;
         len-=max;
      }
      return;
   }

//-------------------------------//


//-------------------------


// End class Alignment
// ********************************* //


