/*
 * Copyright (C) 1999  Piero Fariselli
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * contacts: Piero Fariselli
 *           e-mail: piero@biocomp.unibo.it, farisel@alma.unibo.it
 *           Dept. of Biology
 *           University of Bologna
 *           via Irnerio 42
 *           40126 Bologna
 *           Italy
 */
/** classe Alignment e' fatta per poter allineare 2
sequenze con uno dei metodi disponibili 
questa classe e' stata estesa per utilizzare sequenze che
contengono informazioni anche sulle strutture secondarie, in particolare
SS=strutture secondarie*/

#include <stdio.h>
//#include <iostream.h>
#include <iostream>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <fstream>
#include "Sequence.H"


class Alignment {
  /* -------- general -------------*/
  private :
   Sequence S1;      // sequence 1
   Sequence S2;      // sequence 2
   Sequence S1aligned;      // aligned S1
   Sequence S2aligned;      // aligned S2
//   double weight=1.0;  
   double weight;
   int *seqNum1;     // S1 in ints 
   int *seqNum2;     // S2 in ints 
   int *SS1;        // S1 secondary str  
   int *SS2;        // S2 secondary str
   static const int maxIndexMatSS = 10; // max number of SS 0-9 
   int ** matSS;    // SS score matrix 
   int maxR;           // row length S1+1
   int maxC;           // column length S2+1
   int bestR;          // row where we find the best score end gaps not weighted
   int bestC;          // column where we find the best score end gaps not weighted
   static const int DIMSYMBOL= 23; // residue symbols
   static const char COL='C'; // column move
   static const char DIA='D'; // diagonal move
   static const char ROW='R'; // row move
   static const char ZERO='0';
//   double gap=- 1.0;    
   double gap;    // simple gap
   static const char baseSymbol='0'; // symbol used for the not amplified gaps
   static const char gapSymbol='-'; // gap symbol
//   double gapOpen=-12.0;     // 
   double gapOpen;         // gap open
//   double gapExt=-6.0;      // 
   double gapExt;          // gap extension
//   double ssGapOpen=2.5;    // 
   double ssGapOpen;    // gapopen SS = ssGapOpen* gapOpen for all but baseSymbol
   int ** matS;  // read score matrix
   char * matSname;  // read score matrix name  
   double ** matMR; // match-replace matrix
   double ** matI;  // Insertion 
   double ** matD;  // Deletion
   char *** matBack; // backward matrix

   
// ************** private methods ***********************

   double gapOpenf(char c1, char c2) ;   

   double matchSimMat(int i, int j) ;

   double matchSim(int i, int j) ;

   char max(double c, double d, double r) ;


//-------------------------


   
   int ** setIdentity() ;


   int **  setMatSS() ;

   int ** setPam250() ;
   
   int ** setBlosum62() ;

   int * SeqByte(Sequence s, int len) ;

   int * SeqByteSS(char * s, int lenS, int len) ;

   void mkAlignedSequences (int br, int bc);

   void mkAlignedSequences ();

//##################################//
//  public methods //
   public:
   
// builders
   Alignment(Sequence s1, Sequence s2) ;

   ~Alignment() ;

// *********** other methods  ************
/////   Alignment methods ////////////////

//------------------------------------------//                            

   /** global max similarity */
   double gotohAlSim() ;

   /** global max similarity using weighted gap */
   double gotohAlSimGapWeight() ;

   /** semiglobal max similarity without weighting the gaps at the ends*/
   double gotohAlSimGapWeight0() ;

// parameter settings //

  /** read matrix in the order
       A R N D C Q E G H I  L  K  M  F  P  S  T  W  Y  V B Z X
       0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 2 6 0
     */
   int ** readResidueMatrix(char * filename);

   /**  read matrix in the order
        C H E ? ? ?   
        0 1 2 3 4 - ..  
   */
   int ** readSSMatrix(char * filename) ;


   void setWeight(double w) ;

   void setScoreMatrix(char * s) ;

   void setGap(double gapOpen, double gapExt) ;

   void setGapExt(double gap) ;

   void setSSGapOpen(double gap) ;

   void setGapOpen(double gap) ;

   void setGap(double gap) ;

// *************** prints **************************

   
   double printScore() ;

   void printMatIns() ;

   void printMatDel() ;

   void printMatVal() ;

   void printMatBack() ;

   void printMapSS() ;

   void printAllAlignment(int max) ;

   void printAlignment(int max) ;


};
// END class Alignment
// ********************************* //


