/*
 * Copyright (C) 1999  Piero Fariselli
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * contacts: Piero Fariselli
 *           e-mail: piero@biocomp.unibo.it, farisel@alma.unibo.it
 *           Dept. of Biology
 *           University of Bologna
 *           via Irnerio 42
 *           40126 Bologna
 *           Italy
 */
#include "Sequence.H"
using namespace std;

   /** empty builder */
Sequence::Sequence() {
      name="";
      seq="";
      SS="";
      len=0;
   }
   /** only sequence builder */
Sequence::Sequence(char * s) {
      seq = new char[strlen(s)+1];
      name="";
      strcpy(seq,s); 
      SS="";
      len=strlen(s);
   }
   /** name and sequence builder */
Sequence::Sequence(char * n, char * s) {      
      name = new char[strlen(n)+1];
      seq = new char[strlen(s)+1];
      strcpy(name,n);
      strcpy(seq,s);
      SS="";
      len=strlen(s);
   }
   /** complete builder */
Sequence::Sequence(char * n, char * s, char * ss) {      
      name = new char[strlen(n)+1];
      seq = new char[strlen(s)+1];
      SS = new char[strlen(ss)+1];
      strcpy(name,n);
      strcpy(seq,s);
      strcpy(SS,ss);
      len=strlen(s);
   }

Sequence::~Sequence() {
//	delete name;
//	delete seq;
//	delete SS;
}
// Several methods

   /** set the sequence name */
   void Sequence::setSequenceName(char * n) {
      strcpy(name,n);
   }

   /** set the sequence */
   void  Sequence::setSequence(char * s) {
        strcpy(seq,s);
	len=strlen(s);
   }

   /** set the sequence and name */
   void  Sequence::setSequence(char * n, char * s) {
         strcpy(name,n);
         strcpy(seq,s);
	 len=strlen(s);
   }

   /** set all*/
   void  Sequence::setSequence(char * n, char * s, char * ss) {
         strcpy(name,n);
         strcpy(seq,s);
	 strcpy(SS,ss);
         len=strlen(s);
   }

   /** copy the sequence content of s in another sequence*/
   void  Sequence::cpSequence(Sequence s) {
         strcpy(name,s.name);
         strcpy(seq,s.seq);
	 strcpy(SS,s.SS);
         len=s.len;
   }

   /** return the sequence */
   char *  Sequence::getSequence () {
      return seq;
   }

   /** return the structure  */
   char *  Sequence::getStringSS () {
      return SS;
   }

   /** return the lenght of the structure */
   int  Sequence::getLengthSS () {
      return (int)strlen(SS); 
   }


   /** return the sequence name */
   char *  Sequence::getName() {
      return name;
   }

   /** return the sequence length */
   int  Sequence::getLen() {
      return len;
   }

   /** return the character i of the sequence */
   char  Sequence::charAt(int i) {
      return seq[i]; 
   }

   /** riturn the character i of the structure */
   char  Sequence::ssAt(int i) {
      return SS[i];
   }

   /** return a sequence of only allowed residue symbols
       for the structure only 
       0HECTGSB3XD in SS*/
   Sequence  Sequence::cleanSequence() {
      int i,k,lenss;
      Sequence s =  Sequence(this->name,this->seq,this->SS);
      
      s.cleanSequenceRes();
      k=0;
      lenss=this->getLengthSS();
      for(i=0; i < lenss; i++) {
          switch (this->ssAt(i)) {
            case '0' : s.SS[k++]='0';  break;
            case 'H' : s.SS[k++]='H';  break;
            case 'E' : s.SS[k++]='E';  break;
            case 'C' : s.SS[k++]='C';  break;
            case 'T' : s.SS[k++]='T';  break;
            case 'G' : s.SS[k++]='G';  break;
            case 'S' : s.SS[k++]='S';  break;
            case 'B' : s.SS[k++]='B';  break;
            case '3' : s.SS[k++]='3';  break;
            case 'X' : s.SS[k++]='X';  break;
            case 'D' : s.SS[k++]='D';  break;
          default :  break;
         }
      }
      s.SS[k++]='\0';
      this->cpSequence(s);
      return(s);
   }



   /** return a sequence of only allowed residue symbols
       */
   Sequence  Sequence::cleanSequenceRes() {
      int i,k;
      Sequence s =  Sequence(this->name,this->seq,this->SS);
      k=0;
      for(i=0; i < this->len; i++) {
         switch (this->charAt(i)) {
            case 'A' : s.seq[k++]='A'; break;
            case 'C' : s.seq[k++]='C'; break;
            case 'D' : s.seq[k++]='D'; break;
            case 'E' : s.seq[k++]='E'; break;
            case 'F' : s.seq[k++]='F'; break;
            case 'G' : s.seq[k++]='G'; break;
            case 'H' : s.seq[k++]='H'; break;
            case 'I' : s.seq[k++]='I'; break;
            case 'K' : s.seq[k++]='K'; break;
            case 'L' : s.seq[k++]='L'; break;
            case 'M' : s.seq[k++]='M'; break;
            case 'N' : s.seq[k++]='N'; break;
            case 'P' : s.seq[k++]='P'; break;
            case 'Q' : s.seq[k++]='Q'; break;
            case 'R' : s.seq[k++]='R'; break;
            case 'S' : s.seq[k++]='S'; break;
            case 'T' : s.seq[k++]='T'; break;
            case 'V' : s.seq[k++]='V'; break;
            case 'W' : s.seq[k++]='W'; break;
            case 'Y' : s.seq[k++]='Y'; break;
            case 'B' : s.seq[k++]='B'; break; // residui anomali
            case 'X' : s.seq[k++]='X'; break; //  residui anomali
          default :  break;
         }
      }
      s.seq[k++]='\0';
      s.len=strlen(s.seq);
      this->cpSequence(s);
      return(s);
   }


// print methods

   /** print as it is */
   void  Sequence::printSequence() {
      cout << "> " << name << " len " << len << endl;
      cout << seq << endl;
   }

   /** print the sequence in column of di length max */
   void  Sequence::printSequence(int max) {
      int i;
       cout << ">"<< name <<" len "<< len << endl;

      for(i=0; i< this->len; i++)
      {
         if((i+1)%max == 0) 
            cout << this->charAt(i)<< endl;
         else 
            cout << this->charAt(i);
      }
      if(this->len%max != 0)
         cout <<endl;;

   }

   /** print all the sequence features as they are */
   void  Sequence::printSequenceAll() {
       cout << ">"<< name <<" len "<< len << endl;
      cout << seq << endl;
      cout << this->SS << endl;
      cout << "==============================="<< endl;;
   }
// END class Sequence
// ********************************* //
