/*
 * Copyright (C) 1999  Piero Fariselli
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * contacts: Piero Fariselli
 *           e-mail: piero@biocomp.unibo.it, farisel@alma.unibo.it
 *           Dept. of Biology
 *           University of Bologna
 *           via Irnerio 42
 *           40126 Bologna
 *           Italy
 */
#include "yap.H"
using namespace std;
using std::cout;
using std::ifstream;

int main(int argc, char *argv[])  // 
{
   int i;
   char c;
   int numColAlignment=60;
   char EndGaps = 'Y';
   Sequence * seq;
   Alignment *aln;

  // if there are input problem print help
   if((argc ==1)||(!(argc%2))) {
	   printHelp(argv[0]);
	   return(0);
   }	   
   for(i=1; i <argc; i+=2)   {       // check the input file 
      if(strcmp(argv[i],"-i")==0) { //
          if((seq=getSequences(argv[i+1]))==NULL) {  // try open it
      	     cout << "ERROR in getSequences\n";
             return(-1);
	  } 
	  else {
	     aln = new Alignment(seq[0],seq[1]); // if OK allocate alignment
	  }
      }
   }  
   if((i>argc)&&(strcmp(argv[i-2],"-i")!=0))  { // test input
	    printHelp(argv[0]);
	    return(0);
   }

    // input parameter evaluation	       
   for(i=1; i <argc; i+=2) {
      c=argv[i][1];	    

      switch (c) {
	     case 'i' : break;
             case 'm' : aln->readResidueMatrix(argv[i+1]); break;
	     case 'M' : aln->setScoreMatrix(argv[i+1]); break;		
	     case 's' : aln->readSSMatrix(argv[i+1]); break;
	     case 'g' : aln->setGapOpen(atof(argv[i+1])); break;		
	     case 'e' : aln->setGapExt(atof(argv[i+1])); break;		
	     case 't' : aln->setSSGapOpen(atof(argv[i+1])); break;		
	     case 'w' : aln->setWeight(atof(argv[i+1])); break;		
	     case 'c' : numColAlignment=atoi(argv[i+1]); break;		
	     case 'E' : EndGaps=toupper(argv[i+1][0]); break;		
	     default : 
		       cout << " don't know option " << argv[i] << "\n";
//	               printHelp(argv[0]);
	               return(0);
      }
   }      
      if(EndGaps == 'N') { // compute and-gap no cost alignment 
         cout << "Alignment computed without weighting the end gaps\n" ;
         aln->gotohAlSimGapWeight0();
      } else {
         cout << "Alignment computed weighting the end gaps\n" ;
         aln->gotohAlSimGapWeight();
      }
      aln->printScore();
      aln->printAllAlignment(numColAlignment);
      /*      
      cout << "MATDEL\n";
      aln->printMatDel(); 
      cout << "MATIS\n";
      aln->printMatIns(); 
      cout << "MATVal\n";
      aln-> printMatVal();
      */	      
}

/** read the extended fasta file*/
Sequence * getSequences(char * filename) {
   const int MAXLINE = 10000;
   char line[MAXLINE];
   char name[MAXLINE];
   char s[MAXLINE]; 
   char ss[MAXLINE];
   Sequence * vseq;
   ifstream f ;
   char * indx;
   int i,nseq;
   int flagRead =0;

   nseq=2; // forced the sequences to be 2 27/01/2006
   f.open(filename);
   if(!f) {
     cout << "Can't open file \n";
     return (NULL);
   }
 /*
   do {
      f.getline(line,MAXLINE);
      if((indx=strchr(line,'>'))!=NULL)
         nseq++;
   }while(!f.eof());
   f.close();
*/
//   f.open(filename);
   vseq= new Sequence [nseq];
   if(!f) {
//   if(! f.is_open()) {
     cout << "Can't open file again " << filename <<"\n";
     return (NULL);
   }
   i=0;
   s[0]=ss[0]=name[0]='\0';
   while(!f.eof()) {
      f.getline(line,MAXLINE);
      if((indx=strchr(line,'>'))!=NULL) { 
         if(strlen(name)>0) {
            vseq[i-1]=Sequence((char *)name,(char *)s,(char *)ss);
            vseq[i-1].cleanSequence();
//            vseq[i-1].printSequenceAll();
         }
         indx++;
         sprintf(name,"%s",indx);
//         cout << name <<  " " << nseq << endl;
         i++;
         flagRead =0;
         s[0]=ss[0]='\0';
      }
      else {
         if((indx=strchr(line,'#'))!=NULL) { 
             flagRead =1;
         }
         switch (flagRead) {
             case 0 : strcat(s,line); break;
             case 1 : strcat(ss,line); break;
         }
      }
   }
   if(strlen(s)>0) {
      vseq[i-1]=Sequence(name,s,ss);
      vseq[i-1].cleanSequence();
//      vseq[i-1].printSequenceAll();
   }
   f.close();   
   return(vseq);
}
//-----------------------------//


//-----------------------------//
void printHelp(char * pname) {

   cout << "This program reads a file with 2 sequences in extended fasta format\n";
   cout << "and returns to the standard output an alignment of the two sequences based on\n";
   cout << "the residue sequences and/or secondary structures (depending on the weight -w option)\n";
   cout << "#\n";
   cout << " syntax : " <<pname <<" Options \n";
   cout << " Options:\n-i <inputfile> | file name \n-m <scorematrix> | file name \n-s <secondary structure score matrix> | filename \n";
   cout << "-g <open gap values> (double)\n-e <gap extention> (double)\n-t <times gap open for secondary structure != Loop> (>=1.0)\n";
   cout << "-w <weight for residue score/secondary structure score> (1.0<= w <=0.0)\n-c <number or column in the alignment> (int)\n";  
   cout << "-M <Flag for defined score Matrix> (I = Identity, P = Pam250, B = Blosum62)\n";  
   cout << "-E <Flag for weighting ending gaps> (Y/N (default Y)\n";  
   cout << "#\n";
   return;	
}
//-----------------------------//
